@@ -36,7 +36,6 @@
 #include <glib/gi18n-lib.h>
 #include <list>
 #include <cstring>
-#include <sstream>
 
 using namespace gcu;
 
@@ -44,7 +43,8 @@ namespace gcp {
 
 Arrow::Arrow (TypeId Type):
 	Object (Type),
-	gccv::ItemClient ()
+	gccv::ItemClient (),
+	m_MinLength (0.)
 {
 	m_Start = m_End = NULL;
 }
@@ -197,36 +197,12 @@ double Arrow::GetYAlign ()
 	return m_y + m_height / 2.;
 }
 
-std::string Arrow::GetProperty (unsigned property) const
-{
-	std::ostringstream res;
-	gcu::Document *doc = GetDocument ();
-	switch (property) {
-	case GCU_PROP_ARROW_COORDS:
-		res.precision (12);
-		res << m_x / doc->GetScale () << " " << m_y / doc->GetScale () << " " << (m_x + m_width) / doc->GetScale () << " " << (m_y + m_height) / doc->GetScale ();
-		break;
- 	case GCU_PROP_ARROW_START_ID:
-		if (m_Start)
-			res << m_Start->GetId ();
-		break;
-	case GCU_PROP_ARROW_END_ID:
-		if (m_End)
-			res << m_End->GetId ();
-		break;
-	default:
-		return Object::GetProperty (property);
-	}
-	return res.str ();
-}
-
 bool Arrow::SetProperty (unsigned property, char const *value)
 {
 	switch (property) {
 	case GCU_PROP_ARROW_COORDS: {
 		double x0, y0, x1, y1;
-		std::istringstream str (value);
-		str >> x0 >> y0 >> x1 >> y1;
+		sscanf (value, "%lg %lg %lg %lg", &x0, &y0, &x1, &y1);
 		gcu::Document *doc = GetDocument ();
 		if (doc) {
 			x0 *= doc->GetScale ();
@@ -237,22 +213,6 @@ bool Arrow::SetProperty (unsigned property, char const *value)
 		SetCoords (x0, y0, x1, y1);
 		break;
 	}
- 	case GCU_PROP_ARROW_START_ID: {
-		gcp::Document *doc = static_cast < gcp::Document * > (GetDocument ());
-		if (doc == NULL)
-			return false;
-		SetStartStep (dynamic_cast < Step * > (doc->GetDescendant (value)));
-		break;
-	}
-	case GCU_PROP_ARROW_END_ID: {
-		gcp::Document *doc = static_cast < gcp::Document * > (GetDocument ());
-		if (doc == NULL)
-			return false;
-		SetEndStep (dynamic_cast < Step * > (doc->GetDescendant (value)));
-		break;
-	}
-	default:
-		return Object::SetProperty (property, value);
 	}
 	return true;
 }
