@@ -4,7 +4,7 @@
  * GChemPaint library
  * reaction-arrow.cc
  *
- * Copyright (C) 2004-2011 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2014 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,6 +29,7 @@
 #include "reaction-prop.h"
 #include "reaction-prop-dlg.h"
 #include "reaction-separator.h"
+#include "reaction-substractor.h"
 #include "step-counter.h"
 #include "document.h"
 #include "settings.h"
@@ -58,11 +59,18 @@ typedef struct {
 class ReactionArrowLine
 {
 public:
+	ReactionArrowLine ();
+
 	unsigned m_nProps;
 	double x, y, width, height, descent, ascent;
 	std::list < ObjPos > m_Props;
 };
 
+ReactionArrowLine::ReactionArrowLine ():
+	m_nProps (0), x (0.), y (0.), width (0.), height (0.), descent (0.), ascent (0.)
+{
+}
+
 class ReactionArrowStep
 {
 public:
@@ -83,6 +91,46 @@ ReactionArrowStep::~ReactionArrowStep ()
 {
 }
 
+class ReactionArrowPrivate {
+public:
+	static void DoInvert (ReactionArrow *arrow);
+	static void DoSwapHeads (ReactionArrow *arrow);
+};
+
+void ReactionArrowPrivate::DoInvert (ReactionArrow *arrow)
+{
+	Document *doc = dynamic_cast<Document*> (arrow->GetDocument ());
+	Operation *op = doc->GetNewOperation (GCP_MODIFY_OPERATION);
+	Object *group = arrow->GetGroup ();
+	if (group == NULL)
+		group = arrow;
+	op->AddObject (group, 0);
+	Step *buf = arrow->GetStartStep ();
+	arrow->SetStartStep (arrow->GetEndStep ());
+	arrow->SetEndStep (buf);
+	arrow->m_x += arrow->m_width;
+	arrow->m_width = -arrow->m_width;
+	arrow->m_y += arrow->m_height;
+	arrow->m_height = -arrow->m_height;
+	op->AddObject (group, 1);
+	doc->FinishOperation ();
+	doc->GetView ()->Update (arrow);
+}
+
+void ReactionArrowPrivate::DoSwapHeads (ReactionArrow *arrow)
+{
+	Document *doc = dynamic_cast<Document*> (arrow->GetDocument ());
+	Operation *op = doc->GetNewOperation (GCP_MODIFY_OPERATION);
+	Object *group = arrow->GetGroup ();
+	if (group == NULL)
+		group = arrow;
+	op->AddObject (group, 0);
+	arrow->m_Type = (arrow->m_Type == ReversibleArrow)? FullReversibleArrow: ReversibleArrow;
+	op->AddObject (group, 1);
+	doc->FinishOperation ();
+	doc->GetView ()->Update (arrow);
+}
+
 ReactionArrow::ReactionArrow (Reaction* react, unsigned Type): Arrow (ReactionArrowType)
 {
 	SetId ("ra1");
@@ -181,6 +229,8 @@ bool ReactionArrow::Load (xmlNodePtr node)
 		// and no missing lines steps or ranks
 		// FIXME: write that code!
 
+		// ensure that everything is added to the canvas
+		static_cast < Document * > (doc)->GetView ()->AddObject (this);
 		parent = GetParent ();
 		if (!parent)
 			return true;
@@ -371,47 +421,52 @@ bool ReactionArrow::BuildContextualMenu (gcu::UIManager *UIManager, Object *obje
 	Document *Doc = dynamic_cast<Document*> (GetDocument ());
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (Doc->GetWidget ()), "data");
 	// If there are several lines of objects attached to the arrow, add a property dialog
-	bool result = false;
 	GtkActionGroup *group = NULL;
-	GtkAction *action;
+	group = gtk_action_group_new ("reaction-arrow");
+	GtkAction *action = gtk_action_new ("Arrow", _("Arrow"), NULL, NULL);
+	gtk_action_group_add_action (group, action);
+	g_object_unref (action);
 	if (GetChildrenNumber () > 0) {
-		group = gtk_action_group_new ("reaction-arrow");
-		GtkAction *action = gtk_action_new ("Arrow", _("Arrow"), NULL, NULL);
-		gtk_action_group_add_action (group, action);
-		g_object_unref (action);
 		action = gtk_action_new ("props", _("Properties..."), _("Arrow properties"), NULL);
 		gtk_action_group_add_action (group, action);
 		g_object_unref (action);
 		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_props), this);
 		gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Arrow'><menuitem action='props'/></menu></popup></ui>", -1, NULL);
 		gtk_ui_manager_insert_action_group (uim, group, 0);
-		result = true;
 	}
 	if (pData->SelectedObjects.size () == 1) {
 		Object *obj = *pData->SelectedObjects.begin ();
-		TypeId Id = obj->GetType ();
-		if ((Id != MoleculeType && Id != TextType) || obj->GetGroup ())
-			return Object::BuildContextualMenu (UIManager, object, x, y);
-		if (group == NULL) {
-			group = gtk_action_group_new ("reaction-arrow");
-			action = gtk_action_new ("Arrow", _("Arrow"), NULL, NULL);
+		std::set<TypeId> types;
+		obj->GetPossibleAncestorTypes (types);
+		if (types.find (ReactionPropType) != types.end () && obj->GetGroup () == NULL) {
+			struct CallbackData *data = new struct CallbackData ();
+			data->arrow = this;
+			data->child = obj;
+			action = gtk_action_new ("attach", _("Attach selection to arrow..."), NULL, NULL);
+			g_object_set_data_full (G_OBJECT (action), "data", data, (GDestroyNotify) do_free_data);
+			g_signal_connect_swapped (action, "activate", G_CALLBACK (do_attach_object), data);
 			gtk_action_group_add_action (group, action);
 			g_object_unref (action);
+			gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Arrow'><menuitem action='attach'/></menu></popup></ui>", -1, NULL);
 		}
-		struct CallbackData *data = new struct CallbackData ();
-		data->arrow = this;
-		data->child = obj;
-		action = gtk_action_new ("attach", _("Attach selection to arrow..."), NULL, NULL);
-		g_object_set_data_full (G_OBJECT (action), "data", data, (GDestroyNotify) do_free_data);
-		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_attach_object), data);
+	}
+	action = gtk_action_new ("invert", _("Invert"), _("Invert arrow"), NULL); 
+	gtk_action_group_add_action (group, action);
+	g_object_unref (action);
+	g_signal_connect_swapped (action, "activate", G_CALLBACK (ReactionArrowPrivate::DoInvert), this);
+	gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Arrow'><menuitem action='invert'/></menu></popup></ui>", -1, NULL);
+	if (m_Type != SimpleArrow) {
+		action = gtk_action_new ("swap-heads", ((m_Type == ReversibleArrow)? _("Full heads"): _("Half heads")), _("Swap arrow head style between half and full"), NULL); 
 		gtk_action_group_add_action (group, action);
 		g_object_unref (action);
-		gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Arrow'><menuitem action='attach'/></menu></popup></ui>", -1, NULL);
-		gtk_ui_manager_insert_action_group (uim, group, 0);
-		g_object_unref (group);
-		result = true;
+		g_signal_connect_swapped (action, "activate", G_CALLBACK (ReactionArrowPrivate::DoSwapHeads), this);
+		gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Arrow'><menuitem action='invert'/><menuitem action='swap-heads'/></menu></popup></ui>", -1, NULL);
 	}
-	return result || Object::BuildContextualMenu (UIManager, object, x, y);
+	
+	gtk_ui_manager_insert_action_group (uim, group, 0);
+	g_object_unref (group);
+	Object::BuildContextualMenu (UIManager, object, x, y);
+	return true;
 }
 
 void ReactionArrow::Move (double x, double y, double z)
@@ -475,81 +530,13 @@ void ReactionArrow::AddProp (Object *object)
 	EmitSignal (OnChangedSignal);
 	new ReactionPropDlg (this, prop);
 }
-/*
-void ReactionArrow::PositionChild (ReactionProp *prop)
-{
-	// FIXME: this is experimental code
-	Document *Doc = dynamic_cast<Document*> (GetDocument ());
-	Theme *pTheme = Doc->GetTheme ();
-	double xmin, xspan, ymin, yspan,
-		length = sqrt (m_width * m_width + m_height * m_height),
-		x = m_width / length, y = m_height / length;
-	gccv::Rect rect;
-	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (Doc->GetWidget ()), "data");
-	pData->GetObjectBounds (prop, &rect);
-	if (m_width >=0) {
-		if (m_height >=0) {
-			xmin = (rect.x0 * x + rect.y0 * y) / pTheme->GetZoomFactor ();
-			xspan = (rect.x1 * x + rect.y1 * y) / pTheme->GetZoomFactor () - xmin;
-			ymin = (rect.x0 * y - rect.y1 * x) / pTheme->GetZoomFactor ();
-			yspan = (rect.x1 * y - rect.y0 * x) / pTheme->GetZoomFactor () - ymin;
-		} else {
-			xmin = (rect.x0 * x + rect.y1 * y) / pTheme->GetZoomFactor ();
-			xspan = (rect.x1 * x + rect.y0 * y) / pTheme->GetZoomFactor () - xmin;
-			ymin = (rect.x0 * y - rect.y0 * x) / pTheme->GetZoomFactor ();
-			yspan = (rect.x1 * y - rect.y1 * x) / pTheme->GetZoomFactor () - ymin;
-		}
-	} else {
-		if (m_height >=0) {
-			xmin = (rect.x1 * x + rect.y0 * y) / pTheme->GetZoomFactor ();
-			xspan = (rect.x0 * x + rect.y1 * y) / pTheme->GetZoomFactor () - xmin;
-			ymin = (rect.x1 * y - rect.y1 * x) / pTheme->GetZoomFactor ();
-			yspan = (rect.x0 * y - rect.y0 * x) / pTheme->GetZoomFactor () - ymin;
-		} else {
-			xmin = (rect.x1 * x + rect.y1 * y) / pTheme->GetZoomFactor ();
-			xspan = (rect.x0 * x + rect.y0 * y) / pTheme->GetZoomFactor () - xmin;
-			ymin = (rect.x1 * y - rect.y0 * x) / pTheme->GetZoomFactor ();
-			yspan = (rect.x0 * y - rect.y1 * x) / pTheme->GetZoomFactor () - ymin;
-		}
-	}
-	xspan = fabs (xspan);
-	yspan = fabs (yspan);
-	// xmin and ymin will now be the current center of the object
-	xspan += (2* pTheme->GetArrowObjectPadding () + pTheme->GetArrowHeadA ()) / pTheme->GetZoomFactor ();
-	// adjust the arrow length if needed
-	if (xspan > length) {
-		m_width *= xspan / length;
-		m_height *= xspan / length;
-		length = xspan;
-	}
-	// now move the child to the right place
-	length -= pTheme->GetArrowHeadA () / pTheme->GetZoomFactor ();
-	length /= 2.;
-	// FIXME: using GetArrowDist is a non-sense, we should have a new variable.
-	yspan = yspan / 2. + pTheme->GetArrowDist () / pTheme->GetZoomFactor ();
-	// calculate the vector of the needed move
-	xmin = m_x + length * x + y * yspan - (rect.x0 + rect.x1) / 2. / pTheme->GetZoomFactor ();
-	ymin = m_y + length * y - x * yspan - (rect.y0 + rect.y1) / 2. / pTheme->GetZoomFactor ();
-	prop->Move (xmin, ymin);
-	Doc->GetView ()->Update (this);
-}*/
-/*
-typedef struct {
-	ObjPos *objs;
-	double *ascents;
-	unsigned max;
-	double width, height, ascent, descent;
-} Line;
-
-typedef struct {
-	Line *lines;
-	unsigned max;
-} ArrowStep;*/
 
 void ReactionArrow::PositionChildren ()
 {
+	if (GetParent () == NULL) return;
 	std::vector < StepCounter * > counters;
 	std::set < ReactionSeparator * > separators;
+	std::set < ReactionSubstractor * > substractors;
 	std::set < gcu::Object * > garbage;
 	std::map < std::string, gcu::Object * >::iterator i;
 	gcu::Object *obj = GetFirstChild (i);
@@ -558,11 +545,12 @@ void ReactionArrow::PositionChildren ()
 	Theme const *theme = doc->GetTheme ();
 	WidgetData* data = reinterpret_cast < WidgetData * > ( g_object_get_data (G_OBJECT (doc->GetWidget ()), "data"));
 	gccv::Rect rect;
-	double y, ascent, descent, sep_width = 0., sep_ascent = 0., sep_descent = 0.,
+	double y, ascent, descent, sep_width = 0., sep_ascent = 0., sep_descent = 0., subst_width = 0.,
 		   counter_width = 0., counter_ascent = 0., counter_descent = 0.,
 		   uwidth = 0., uheight = 0., lwidth = 0., lheight = 0.;
-	bool needs_sep = false;
+	bool needs_sep = false, needs_subst = false;
 	ReactionSeparator *sep;
+	ReactionSubstractor *subst;
 	double scale = theme->GetZoomFactor (), padding = theme->GetPadding ();
 	StepCounter *counter;
 	double lxspan, lyspan, uxspan, uyspan, x, length, xspan, yspan, xmin, ymin, width, xc, yc;
@@ -580,6 +568,8 @@ void ReactionArrow::PositionChildren ()
 	while (obj) {
 		if (obj->GetType () == ReactionSeparatorType)
 			separators.insert (static_cast < ReactionSeparator * > (obj));
+		if (obj->GetType () == ReactionSubstractorType)
+			substractors.insert (static_cast < ReactionSubstractor * > (obj));
 		else if (obj->GetType () == StepCounterType) {
 			counter = static_cast < StepCounter * > (obj);
 			cur = counter->GetStep () - 1;
@@ -590,6 +580,8 @@ void ReactionArrow::PositionChildren ()
 		} else if (obj->GetType () == ReactionPropType) {
 			if (!needs_sep && static_cast < ReactionProp * > (obj)->GetRank () > 1)
 				needs_sep = true;
+			if (!needs_subst && static_cast < ReactionProp * > (obj)->GetRole () == REACTION_PROP_PRODUCT)
+				needs_subst = true;
 		}
 		else
 			garbage.insert (obj);
@@ -612,6 +604,17 @@ void ReactionArrow::PositionChildren ()
 		sep_ascent = y - rect.y0 / scale;
 		sep_descent = rect.y1 / scale - y;
 	}
+	if (needs_subst) {
+		if (substractors.empty ()) {
+			subst = new ReactionSubstractor ();
+			substractors.insert (subst);
+			AddChild (subst);
+			doc->GetView ()->AddObject (subst);
+		} else
+			subst = *substractors.begin ();
+		data->GetObjectBounds (subst, &rect);
+		subst_width = (rect.x1 - rect.x0) / scale;
+	}
 	// evaluate step counters size, and create them if needed
 	if (max_step > 1) {
 		for (cur = 0; cur < max_step; cur++) {
@@ -659,6 +662,8 @@ void ReactionArrow::PositionChildren ()
 					line->ascent = ascent;
 				if (line->descent < descent)
 					line->descent = descent;
+				if (static_cast < ReactionProp * > (obj)->GetRole () == REACTION_PROP_PRODUCT)
+					line->width += subst_width;
 			}
 			if (line->m_Props.size () > 1) {
 				line->width += sep_width * (line->m_Props.size () - 1);
@@ -692,8 +697,9 @@ void ReactionArrow::PositionChildren ()
 	lwidth += counter_width;
 	// evaluate needed arrow size
 	if (cur > m_MaxLinesAbove) {
-		uheight += (m_MaxLinesAbove - 1) * padding;
-		cur ++;
+		if (m_MaxLinesAbove > 0)
+			uheight += (m_MaxLinesAbove - 1) * padding;
+		cur --;
 		if (cur > m_MaxLinesAbove)
 			lheight = (cur - m_MaxLinesAbove) * padding;
 	} else
@@ -706,6 +712,8 @@ void ReactionArrow::PositionChildren ()
 	lyspan = fabs (lwidth * y  + lheight * x);
 	uxspan = fabs (uwidth * x + uheight * y);
 	uyspan = fabs (uwidth * y  + uheight * x);
+	x = m_width / length;
+	y = m_height / length;
 	xspan = (uxspan > lxspan)? uxspan: lxspan,
 	xspan += (2* theme->GetArrowObjectPadding () + theme->GetArrowHeadA ()) / theme->GetZoomFactor ();
 	// adjust the arrow length if needed
@@ -757,7 +765,6 @@ void ReactionArrow::PositionChildren ()
 					// insert a separator
 					if (separators.empty ()) {
 						sep = new ReactionSeparator ();
-						separators.insert (sep);
 						AddChild (sep);
 						doc->GetView ()->AddObject (sep);
 					} else {
@@ -767,6 +774,20 @@ void ReactionArrow::PositionChildren ()
 					sep->SetCoords (xc, yc);
 					xc += sep_width;
 				}
+				if (static_cast < ReactionProp * > (obj)->GetRole () == REACTION_PROP_PRODUCT) {
+					// insert a separator
+					if (substractors.empty ()) {
+						subst = new ReactionSubstractor ();
+						substractors.insert (subst);
+						AddChild (subst);
+						doc->GetView ()->AddObject (subst);
+					} else {
+						subst = *substractors.begin ();
+						substractors.erase (subst);
+					}
+					subst->SetCoords (xc, yc);
+					xc += subst_width;
+				}
 				(*ip).obj->Move (xc - (*ip).x, yc - (*ip).y);
 				xc += (*ip).width;
 			}
@@ -916,7 +937,6 @@ void ReactionArrow::OnLoaded ()
 	// remove garbage
 	std::set < gcu::Object * >::iterator j, jend = garbage.end ();
 	for (j = garbage.begin (); j != jend; j++) {
-printf("deleting object %p\n",*j);
 		if ((*j)->GetType () == ReactionPropType)
 			static_cast < ReactionProp * > (*j)->GetObject ()->SetParent (doc);
 		else
@@ -1217,4 +1237,10 @@ void ReactionArrow::RemoveProp (ReactionProp *prop)
 	}
 }
 
+void ReactionArrow::SetCoords (double xstart, double ystart, double xend, double yend)
+{
+	Arrow::SetCoords (xstart, ystart, xend, yend);
+	PositionChildren ();
+}
+
 }	//	namespace gcp
