@@ -52,6 +52,25 @@ static void help_cb (Tools *box)
 	box->OnHelp ();
 }
 
+Tools::Tools (Application *App):
+	gcugtk::Dialog (App, UIDIR"/tools.ui", "tools", GETTEXT_PACKAGE, App),
+	m_UIManager (NULL)
+{
+	g_signal_connect (G_OBJECT (dialog), "delete-event", G_CALLBACK (on_delete_event), NULL);
+	Application *pApp = dynamic_cast<Application*> (App);
+	m_UIManager = NULL;
+	m_ButtonsGrid = GTK_GRID (GetWidget ("tools-buttons"));
+	m_Book = GTK_NOTEBOOK (GetWidget ("tools-book"));
+	GtkWidget *grid = GetWidget ("element-grid");
+	GtkWidget *w = gcu_combo_periodic_new ();
+	m_Mendeleiev = reinterpret_cast <GcuComboPeriodic *> (w);
+	gtk_container_add (GTK_CONTAINER (grid), w);
+	gcu_combo_periodic_set_element (GCU_COMBO_PERIODIC (w), pApp->GetCurZ ());
+	g_signal_connect_swapped (G_OBJECT (w), "changed", G_CALLBACK (element_changed_cb), this);
+	w = GetWidget ("help-btn");
+	g_signal_connect_swapped (G_OBJECT (w), "clicked", G_CALLBACK (help_cb), this);
+}
+
 class Toolbar {
 public:
 	Toolbar ();
@@ -73,7 +92,7 @@ static void tool_toggled_cb (GtkToggleToolButton *btn, Application *app)
 
 Tools::Tools (Application *app, std::list < ToolDesc const * > const &descs):
 	gcugtk::Dialog (app, UIDIR"/tools.ui", "tools", GETTEXT_PACKAGE, app),
-	m_Parent (NULL)
+	m_UIManager (NULL)
 {
 	std::list < ToolDesc const * >::const_iterator d, end = descs.end ();
 	std::vector < Toolbar > toolbars;
@@ -151,11 +170,14 @@ Tools::Tools (Application *app, std::list < ToolDesc const * > const &descs):
 	w = GetWidget ("help-btn");
 	g_signal_connect_swapped (G_OBJECT (w), "clicked", G_CALLBACK (help_cb), this);
 	g_signal_connect (G_OBJECT (dialog), "delete-event", G_CALLBACK (on_delete_event), NULL);
-	gtk_widget_show_all (GTK_WIDGET (dialog));
 }
 
 Tools::~Tools ()
 {
+	if (m_UIManager) {
+		delete m_UIManager;
+		m_UIManager = NULL;
+	}
 }
 
 void Tools::Show (bool visible)
@@ -174,6 +196,23 @@ void register_item_cb (GtkWidget *w, Tools *Dlg)
 	Dlg->RegisterTool (w);
 }
 
+void Tools::AddToolbar (string &name)
+{
+	if (m_UIManager) {
+		GtkWidget *w = gtk_ui_manager_get_widget (m_UIManager->GetUIManager (), name.c_str ());
+		gtk_container_foreach (GTK_CONTAINER (w), (GtkCallback) register_item_cb, this);
+		gtk_toolbar_set_style (GTK_TOOLBAR (w), GTK_TOOLBAR_ICONS);
+		gtk_toolbar_set_show_arrow (GTK_TOOLBAR (w), false);
+		gtk_container_add (GTK_CONTAINER (m_ButtonsGrid), w);
+		gtk_widget_show_all (w);
+	}
+}
+
+void Tools::SetUIManager (gcugtk::UIManager *manager)
+{
+	m_UIManager = manager;
+}
+
 void Tools::SetPage (Tool *tool, int i)
 {
 	m_Pages[tool] = i;
