@@ -51,19 +51,13 @@ using namespace std;
 static void on_flip (GtkWidget *btn, gcp::Application* App)
 {
 	gcpSelectionTool *tool = (gcpSelectionTool*) App->GetTool ("Select");
-	if (GTK_IS_WIDGET (btn))
-		tool->OnFlip (strcmp (gtk_widget_get_name (btn), "VertFlip"));
-	else
-		tool->OnFlip (strcmp (gtk_action_get_name (GTK_ACTION (btn)), "VertFlip"));
+	tool->OnFlip (strcmp (gtk_widget_get_name (btn), "VertFlip"));
 }
 
-static void on_rotate (GtkWidget *btn, gcp::Application* App)
+static void on_rotate (GtkToggleToolButton *btn, gcp::Application* App)
 {
 	gcpSelectionTool *tool = (gcpSelectionTool*) App->GetTool ("Select");
-	if (GTK_IS_WIDGET (btn))
-		tool->Rotate (gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON (btn)));
-	else
-		tool->Rotate (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (btn)));
+	tool->Rotate (gtk_toggle_tool_button_get_active (btn));
 }
 
 static void on_merge (G_GNUC_UNUSED GtkWidget *btn, gcp::Application* App)
@@ -283,7 +277,7 @@ void gcpSelectionTool::AddSelection (gcp::WidgetData* data)
 			m_pData = d;
 		}
 		// If the selection is made of two molecules, activate the merge tool
-		if (m_UIManager) {
+		if (m_MergeBtn) {
 			std::set < Object * >::iterator i = m_pData->SelectedObjects.begin ();
 			gtk_widget_set_sensitive (m_MergeBtn, ((m_pData->SelectedObjects.size () == 2) &&
 				((*i++)->GetType () == MoleculeType) &&
